/**
 * gclc:fr.bigeon.gclc.tools.AOutputForwardRunnable.java
 * Created on: Dec 1, 2016
 */
package fr.bigeon.gclc.tools;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import fr.bigeon.gclc.manager.PipedConsoleManager;

/** An incomplete implematation used to forward messages from a piped console.
 * <p>
 * This forwarding can be interrupted without closing the piped manager.
 *
 * @author Emmanuel Bigeon */
public abstract class AOutputForwardRunnable implements Runnable {

    /** The class logger */
    private static final Logger LOGGER = Logger
            .getLogger(AOutputForwardRunnable.class.getName());
    /** The default timeout (one tenth of second). */
    private static final long DEFAULT_TIMEOUT = 100;
    /** The manager. */
    private final PipedConsoleManager manager;
    /** The timeout */
    private final long timeout;

    /** Create a forward runnable with the given timeout.
     * <p>
     * Short timeout will be very responsive to the application actual messages,
     * but may use computation time if the application is not verbose. Long
     * timeout will save computation time, but will read batches of messages at
     * once if the application is verbose. The right length for the timeout is
     * likely to depend on the application and the use of it.
     * <p>
     * If you do not know what timeout length to use, please use the
     * {@link #AOutputForwardRunnable(PipedConsoleManager)} constructor.
     * 
     * @param manager the manager
     * @param timeout the timeout between message requests. */
    public AOutputForwardRunnable(PipedConsoleManager manager, long timeout) {
        super();
        this.manager = manager;
        this.timeout = timeout;
    }

    /** Create a forwarding runnable.
     * 
     * @param manager the manager */
    public AOutputForwardRunnable(PipedConsoleManager manager) {
        super();
        this.manager = manager;
        timeout = DEFAULT_TIMEOUT;
    }

    @Override
    public void run() {
        try {
            while (isRunning()) {
                while (isRunning() && !manager.available()) {
                    waitASec();
                }
                if (!isRunning()) {
                    return;
                }
                String m = manager.readNextLine();
                forwardLine(m);
            }
        } catch (IOException e) {
            LOGGER.log(Level.SEVERE, "Unexpected problem in manager", //$NON-NLS-1$
                    e);
        }
    }

    /** @param m the line to forward */
    protected abstract void forwardLine(String m);

    /** @return if the thread should keep running */
    protected abstract boolean isRunning();

    /** a method to wait some time */
    protected void waitASec() {
        try {
            synchronized (this) {
                wait(timeout);
            }
        } catch (InterruptedException e) {
            LOGGER.log(Level.SEVERE, "Interrupted wait", //$NON-NLS-1$
                    e);
            return;
        }
    }

}
