/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.command.HelpExecutorTest.java
 * Created on: Nov 19, 2016
 */
package fr.bigeon.gclc.command;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

import org.junit.Test;

import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.manager.PipedConsoleManager;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon
 *
 */
public class HelpExecutorTest {

    /**
     * Test method for {@link fr.bigeon.gclc.command.HelpExecutor#HelpExecutor(java.lang.String, fr.bigeon.gclc.manager.ConsoleManager, fr.bigeon.gclc.command.ICommand)}.
     */
    @Test
    public final void testHelpExecutor(){
        HelpExecutor help;
        try {
            help = new HelpExecutor("?", null, new MockCommand("mock"));
            fail("help is an interactive command, should need console manager");
        } catch (Exception e) {
            assertNotNull(e);
        }
        try (PipedConsoleManager test = new PipedConsoleManager()) {
            help = new HelpExecutor("?", test, new MockCommand("mock"));
        } catch (Exception e) {
            assertNull(e);
        }
    }

    /**
     * Test method for {@link fr.bigeon.gclc.command.HelpExecutor#execute(java.lang.String[])}.
     */
    @Test
    public final void testExecute(){
        try {
            PipedConsoleManager test = new PipedConsoleManager();
            HelpExecutor help = new HelpExecutor("?", test,
                    new Command("mock") {

                        @Override
                        public void execute(String... args) throws CommandRunException {
                            //
                        }

                        @Override
                        public String tip() {
                            return "";
                        }

                    });

            help.execute();
            test.close();

            try {
                help.execute();
                fail("manager closed shall provoke failure of help command execution");
            } catch (Exception e) {
                assertNotNull(e);
            }
        } catch (Exception e) {
            assertNull(e);
        }
    }

    /**
     * Test method for {@link fr.bigeon.gclc.command.HelpExecutor#tip()}.
     */
    @Test
    public final void testTip(){
        try (PipedConsoleManager test = new PipedConsoleManager()) {
            HelpExecutor help = new HelpExecutor("?", test,
                    new MockCommand("mock"));
            help.tip();
            help.help(test);
        } catch (Exception e) {
            assertNull(e);
        }
        try (PipedConsoleManager test = new PipedConsoleManager()) {
            HelpExecutor help = new HelpExecutor("?", test,
                    new SubedCommand("sub", new MockCommand("mock")));
            help.tip();
            help.help(test);
        } catch (Exception e) {
            assertNull(e);
        }
    }

}
